from random import randint, sample, seed, shuffle
seed(47)

def save(fname, D, rooks):
    shuffle(rooks)
    f = open(fname, 'w')
    print(D, len(rooks), file=f)
    for r,c,t in rooks: print(r, c, t, file=f)
    f.close()

def lograndom(maxbits):
    b = randint(1,maxbits)
    return randint(2**(b-1),(2**b)-1)

def gen_random(D, N):
    occupied = set()
    rooks = []
    while len(rooks) < N:
        r = randint(0, D-1)
        c = randint(0, D-1)
        t = "BC"[randint(0,1)]
        if (r, c) in occupied: continue
        occupied.add( (r, c) )
        rooks.append( (r, c, t) )
    return D, rooks

def gen_distinct_rowcol(D, N):
    occupied_r = set()
    occupied_c = set()
    rooks = []
    while len(rooks) < N:
        r = randint(0, D-1)
        c = randint(0, D-1)
        if r in occupied_r: continue
        if c in occupied_c: continue
        t = "BC"[randint(0,1)]
        occupied_r.add(r)
        occupied_c.add(c)
        rooks.append( (r, c, t) )
    return D, rooks

def random_coords(D, count):
    coords = set()
    while len(coords) < count: coords.add( randint(0,D-1) )
    return list(sorted(coords))

def gen_lograndom(D, N, maxbits):
    while True:
        rowcount = lograndom(maxbits)
        colcount = lograndom(maxbits)
        if rowcount * colcount >= N: break
    rows = random_coords(D, rowcount)
    cols = random_coords(D, colcount)

    occupied = set()
    rooks = []
    while len(rooks) < N:
        r = rows[ randint(0, rowcount-1) ]
        c = cols[ randint(0, colcount-1) ]
        t = "BC"[randint(0,1)]
        if (r, c) in occupied: continue
        occupied.add( (r, c) )
        rooks.append( (r, c, t) )
    return D, rooks

def all_white(D, rooks):
    return D, [ (r, c, 'B') for r, c, t in rooks ]

save('00.sample.1.in', 8, [ (4,7,'B') ] )
save('00.sample.2.in', 100002, [ (0,0,'C'), (0,11,'C'), (11,0,'C'), (11,11,'C') ] )
save('00.sample.3.in', *gen_random(7,7))

save('01.1.in', *all_white(*gen_random(1, 0)))
save('01.2.in', *all_white(*gen_random(7, 0)))
save('01.3.in', *all_white(*gen_random(8, 3)))
save('01.4.in', *all_white(*gen_random(8, 12)))
save('01.5.in', *all_white(*gen_random(8, 25)))

save('02.1.in', *gen_distinct_rowcol(2,2))
save('02.2.in', *gen_distinct_rowcol(5,5))
save('02.3.in', *gen_distinct_rowcol(20,14))
save('02.4.in', *gen_distinct_rowcol(20,20))

save('03.1.in', *gen_random(50,5))
save('03.2.in', *gen_random(50,30))
save('03.3.in', *gen_random(50,120))
save('03.4.in', *gen_random(50,1847))

save('04.1.in', *all_white(*gen_random(10**6, 0)))
save('04.2.in', *all_white(*gen_random(10**6, 3)))
save('04.3.in', *all_white(*gen_lograndom(10**6, 22, 3)))
save('04.4.in', *all_white(*gen_lograndom(10**6, 22, 4)))
save('04.5.in', *all_white(*gen_lograndom(10**6, 100, 4)))
save('04.6.in', *all_white(*gen_lograndom(10**6, 100, 5)))
save('04.7.in', *all_white(*gen_random(10**6, 100)))

save('05.1.in', *gen_random(10**6, 0))
save('05.2.in', *gen_random(10**6, 3))
save('05.3.in', *gen_lograndom(10**6, 22, 3))
save('05.4.in', *gen_lograndom(10**6, 22, 4))
save('05.5.in', *gen_lograndom(10**6, 100, 4))
save('05.6.in', *gen_lograndom(10**6, 100, 5))
save('05.7.in', *gen_random(10**6, 100))

save('06.1.in', *all_white(*gen_random(10**6, 10000)))
save('06.2.in', *all_white(*gen_random(10**6, 30000)))
save('06.3.in', *all_white(*gen_random(10**6, 60000)))
save('06.4.in', *all_white(*gen_lograndom(10**6, 60000, 8)))
save('06.5.in', *all_white(*gen_lograndom(10**6, 60000, 9)))
save('06.6.in', *all_white(*gen_lograndom(10**6, 60000, 10)))
save('06.7.in', *all_white(*gen_lograndom(10**6, 60000, 11)))
save('06.8.in', *all_white(*gen_lograndom(10**6, 60000, 13)))

save('07.1.in', *gen_random(10**6, 33333))
save('07.2.in', *gen_random(10**6, 100000))
save('07.3.in', *gen_random(10**6, 200000))
save('07.4.in', *gen_lograndom(10**6, 200000, 9))
save('07.5.in', *gen_lograndom(10**6, 200000, 11))
save('07.6.in', *gen_lograndom(10**6, 200000, 13))

save('08.1.in', *gen_distinct_rowcol(205000,199879))
save('08.2.in', *gen_distinct_rowcol(1234567,198989))
save('08.3.in', *gen_distinct_rowcol(123456789,199909))
save('08.4.in', *gen_distinct_rowcol(987654321,200000))

save('09.1.in', *all_white(*gen_random(10**9, 10000)))
save('09.2.in', *all_white(*gen_random(10**9, 30000)))
save('09.3.in', *all_white(*gen_random(10**9, 60000)))
save('09.4.in', *all_white(*gen_lograndom(10**9, 60000, 8)))
save('09.5.in', *all_white(*gen_lograndom(10**9, 60000, 9)))
save('09.6.in', *all_white(*gen_lograndom(10**9, 60000, 10)))
save('09.7.in', *all_white(*gen_lograndom(10**9, 60000, 11)))
save('09.8.in', *all_white(*gen_lograndom(10**9, 60000, 13)))

save('10.1.in', *gen_random(10**9, 0))
save('10.2.in', *gen_random(10**9, 190000))
save('10.3.in', *gen_random(10**9, 200000))
save('10.4.in', *gen_lograndom(10**9, 200000, 9))
save('10.5.in', *gen_lograndom(10**9, 200000, 10))
save('10.6.in', *gen_lograndom(10**9, 200000, 11))
save('10.7.in', *gen_lograndom(10**9, 200000, 12))
save('10.8.in', *gen_lograndom(10**9, 200000, 13))
save('10.9.in', *gen_lograndom(10**9, 200000, 14))

#    $d$  &  8 & 20 & 50 & $10^6$ & $10^6$ & $10^6$    & $10^6$ & $10^9$ & $10^9$    & $10^9$ \\
#    $n$  &    &    &    & $10^2$ & $10^2$ & $60\,000$ &        &        & $60\,000$ &        \\
# špeciál &  B &  R &    &      B &        &         B &        &      R &         B &        \\
