#include <bits/stdc++.h>
using namespace std;

struct veza {
    int r, c, farba;
    veza(int r, int c, int t) : r(r), c(c), farba(t) {}
};

bool operator<(const veza &A, const veza &B) { if (A.r != B.r) return A.r < B.r; else return A.c < B.c; }

struct fenwick_tree {
    int size;
    vector<int> T;

    fenwick_tree(int maxval) { 
        size=1; while (size < maxval) size <<= 1; T.resize(size+1); 
    }

    void update(int x, int delta) { // pre 1 <= x <= init_maxval
        while (x <= size) { T[x] += delta; x += x & -x; }
    }

    int sum(int x1, int x2) const { // sucet v uzavretom intervale [x1,x2]
        if (x1 > x2) return 0;
        int res=0;
        --x1;
        while (x2 > 0) { res += T[x2]; x2 -= x2 & -x2; }
        while (x1 > 0) { res -= T[x1]; x1 -= x1 & -x1; }
        return res;
    }
};

void vypis(const vector< vector<long long> > &odpoved) {
    for (int b=0; b<5; ++b) for (int c=0; c<5; ++c) if (odpoved[b][c] > 0) {
        cout << b << " " << c << " " << odpoved[b][c] << endl;
    }
}

void sprav_blok(vector< vector<long long> > &odpoved,
                const vector< vector<fenwick_tree> > &FT,
                int clo, int chi, int fvlavo, int fvpravo) {

    for (int fhore=0; fhore<3; ++fhore) for (int fdole=0; fdole<3; ++fdole) {
        vector<int> pocty_farieb(3,0);
        ++pocty_farieb[fvlavo];
        ++pocty_farieb[fvpravo];
        ++pocty_farieb[fhore];
        ++pocty_farieb[fdole];
        int biele = pocty_farieb[1], cierne = pocty_farieb[2];
        odpoved[biele][cierne] += FT[fhore][fdole].sum(clo,chi);
    }
}

int main() {
    // nacitame vstup, rozdelime veze do skupin podla riadkov aj podla stlpcov
    int D, N;
    cin >> D >> N;
    vector< vector<veza> > veze_podla_riadkov(D), veze_podla_stlpcov(D+1);

    for (int n=0; n<N; ++n) {
        int r, c;
        string t;
        cin >> r >> c >> t;
        ++c;
        veze_podla_riadkov[r].emplace_back( r, c, t == "B" ? 1 : 2);
        veze_podla_stlpcov[c].emplace_back( r, c, t == "B" ? 1 : 2);
    }
    
    // vyrobime si pole pre odpovede = pocty policok jednotlivych typov
    vector< vector<long long> > odpoved( 5, vector<long long>(5,0) );
    if (N == 0) { odpoved[0][0] = 1LL*D*D; vypis(odpoved); return 0; }

    // vyrobime si stromy pre vsetky kombinacie farieb
    vector< vector<fenwick_tree> > FT(3, vector<fenwick_tree>(3, fenwick_tree(D)));

    // v kazdom stlpci si pridame na spodok priesvitnu zarazku a usporiadame zdola hore
    for (int c=1; c<=D; ++c) {
        veze_podla_stlpcov[c].emplace_back(D,c,0);
        sort( veze_podla_stlpcov[c].begin(), veze_podla_stlpcov[c].end() );
        reverse( veze_podla_stlpcov[c].begin(), veze_podla_stlpcov[c].end() );
    }
    
    // zatial nastavime ze v kazdom stlpci je farba 0 zhora a farba jeho najvyssej veze zdola
    vector<int> farba_hore(D+1,0);
    for (int c=1; c<=D; ++c) FT[0][ veze_podla_stlpcov[c].back().farba ].update(c,+1);
        
    // v kazdom riadku pridame priesvitne zarazky na oba konce a usortime
    for (int r=0; r<D; ++r) {
        veze_podla_riadkov[r].emplace_back(r,0,0);
        veze_podla_riadkov[r].emplace_back(r,D+1,0);
        sort( veze_podla_riadkov[r].begin(), veze_podla_riadkov[r].end() );
    }

    // postupne prechadzame vsetky riadky
    for (int r=0; r<D; ++r) {
        vector<veza> &akt_veze = veze_podla_riadkov[r];

        // spracujeme bloky prazdnych policok v tomto riadku
        for (unsigned i=0; i+1<akt_veze.size(); ++i) {
            int lo = akt_veze[i].c + 1, hi = akt_veze[i+1].c - 1;
            if (lo > hi) continue;
            sprav_blok(odpoved, FT, lo, hi, akt_veze[i].farba, akt_veze[i+1].farba);
        }

        // upravime data pre stlpce ktore dostali nove veze
        for (unsigned i=1; i+1<akt_veze.size(); ++i) {
            int c = akt_veze[i].c;
            int f1 = farba_hore[c];
            int f2 = akt_veze[i].farba;
            veze_podla_stlpcov[c].pop_back();
            int f3 = veze_podla_stlpcov[c].back().farba;
            FT[f1][f2].update(c,-1);
            FT[f2][f3].update(c,+1);
            farba_hore[c] = f2;
        }
    }
    vypis(odpoved);
}
