#include <bits/stdc++.h>
using namespace std;

struct veza {
    int r, c, farba;
    veza(int r, int c, int t) : r(r), c(c), farba(t) {}
};

bool operator<(const veza &A, const veza &B) { if (A.r != B.r) return A.r < B.r; else return A.c < B.c; }

struct fenwick_tree {
    int size;
    vector<int> T;

    fenwick_tree(int maxval) { 
        size=1; while (size < maxval) size <<= 1; T.resize(size+1); 
    }

    void update(int x, int delta) { // pre 1 <= x <= init_maxval
        while (x <= size) { T[x] += delta; x += x & -x; }
    }

    int sum(int x1, int x2) { // sucet v uzavretom intervale [x1,x2]
        if (x1 > x2) return 0;
        int res=0;
        --x1;
        while (x2 > 0) { res += T[x2]; x2 -= x2 & -x2; }
        while (x1 > 0) { res -= T[x1]; x1 -= x1 & -x1; }
        return res;
    }
};

void vypis(const vector< vector<long long> > &odpoved) {
    for (int b=0; b<5; ++b) for (int c=0; c<5; ++c) if (odpoved[b][c] > 0) {
        cout << b << " " << c << " " << odpoved[b][c] << endl;
    }
}

void sprav_blok(vector< vector<long long> > &odpoved,
                vector< vector<fenwick_tree> > &FT,
                long long nasob, int povodna_dlzka, int clo, int chi, int fvlavo, int fvpravo) {

    int c00 = povodna_dlzka;

    for (int fhore=0; fhore<3; ++fhore) for (int fdole=0; fdole<3; ++fdole) {
        if (fhore == 0 && fdole == 0) continue;
        vector<int> pocty_farieb(3,0);
        ++pocty_farieb[fvlavo];
        ++pocty_farieb[fvpravo];
        ++pocty_farieb[fhore];
        ++pocty_farieb[fdole];
        int biele = pocty_farieb[1], cierne = pocty_farieb[2];
        int kolko = FT[fhore][fdole].sum(clo,chi);
        odpoved[biele][cierne] += nasob * kolko;
        c00 -= kolko;
    }
    {
        vector<int> pocty_farieb(3,0);
        ++pocty_farieb[fvlavo];
        ++pocty_farieb[fvpravo];
        int biele = pocty_farieb[1], cierne = pocty_farieb[2];
        odpoved[biele][cierne] += nasob * c00;
    }
}

int main() {
    // nacitame vstup
    int D, N;
    cin >> D >> N;
    vector<veza> nacitane_veze;
    for (int n=0; n<N; ++n) {
        int r, c;
        string t;
        cin >> r >> c >> t;
        nacitane_veze.emplace_back( r, c, t == "B" ? 1 : 2);
    }
    sort(nacitane_veze.begin(), nacitane_veze.end());
    
    map<int, vector<veza> > veze_podla_riadkov;
    for (auto v : nacitane_veze) veze_podla_riadkov[v.r].push_back(v);

    // spravime kompresiu suradnic pre stlpce
    set<int> rozne_suradnice = {0, D-1}; 
    for (auto v : nacitane_veze) for (int d=-1; d<=1; ++d) if (0 <= v.c+d && v.c+d < D) rozne_suradnice.insert(v.c+d);
    vector<int> suradnice( rozne_suradnice.begin(), rozne_suradnice.end() );
    unordered_map<int,int> redukuj_stlpec;
    for (int i=0; i<int(suradnice.size()); ++i) redukuj_stlpec[suradnice[i]] = i+1;
    int Y = redukuj_stlpec.size();

    // vyrobime si pole pre odpovede = pocty policok jednotlivych typov
    vector< vector<long long> > odpoved( 5, vector<long long>(5,0) );
    if (N == 0) { odpoved[0][0] = 1LL*D*D; vypis(odpoved); return 0; }

    // vyrobime si stromy pre vsetky kombinacie farieb
    vector< vector<fenwick_tree> > FT(3, vector<fenwick_tree>(3, fenwick_tree(Y+1)));

    // roztriedime si veze podla stlpcov
    // v kazdom stlpci si pridame na spodok priesvitnu zarazku a usporiadame zdola hore
    vector< vector<veza> > veze_podla_stlpcov(Y+1);
    for (auto v : nacitane_veze) veze_podla_stlpcov[ redukuj_stlpec[v.c] ].push_back(v);
    for (int y=1; y<=Y; ++y) {
        veze_podla_stlpcov[y].emplace_back(D,-47,0);
        reverse( veze_podla_stlpcov[y].begin(), veze_podla_stlpcov[y].end() );
    }
    
    // zatial nastavime ze v kazdom stlpci je farba 0 zhora a farba jeho najvyssej veze zdola
    vector<int> farba_hore(Y+1,0);
    for (int y=1; y<=Y; ++y) FT[0][ veze_podla_stlpcov[y].back().farba ].update(y,+1);
        
    // postupne prechadzame vsetky riadky
    int pred_riadok = -1;
    for (auto &rec : veze_podla_riadkov) {
        int akt_riadok = rec.first;
        vector<veza> &akt_veze = rec.second;

        // pridame priesvitne zarazky na konce
        akt_veze.insert( akt_veze.begin(), {akt_riadok,-1,0} );
        akt_veze.insert( akt_veze.end(),   {akt_riadok,D,0} );

        // spracujeme prazdne riadky pred aktualnym
        sprav_blok(odpoved, FT, akt_riadok-pred_riadok-1, D, redukuj_stlpec[0], redukuj_stlpec[D-1], 0, 0);
        pred_riadok = akt_riadok;

        // spracujeme bloky prazdnych policok v tomto riadku
        for (unsigned i=0; i+1<akt_veze.size(); ++i) {
            int lo = akt_veze[i].c + 1, hi = akt_veze[i+1].c - 1;
            if (lo > hi) continue;
            sprav_blok(odpoved, FT, 1, hi-lo+1, redukuj_stlpec[lo], redukuj_stlpec[hi], akt_veze[i].farba, akt_veze[i+1].farba);
        }

        // upravime data pre stlpce ktore dostali nove veze
        for (unsigned i=1; i+1<akt_veze.size(); ++i) {
            int c = redukuj_stlpec[ akt_veze[i].c ];
            int f1 = farba_hore[c];
            int f2 = akt_veze[i].farba;
            veze_podla_stlpcov[c].pop_back();
            int f3 = veze_podla_stlpcov[c].back().farba;
            FT[f1][f2].update(c,-1);
            FT[f2][f3].update(c,+1);
            farba_hore[c] = f2;
        }
    }
    if (pred_riadok < D-1) sprav_blok(odpoved, FT, D-pred_riadok-1, D, redukuj_stlpec[0], redukuj_stlpec[D-1], 0, 0);
    vypis(odpoved);
}
